/**
 * Written by David Henningsson <david.henningsson@canonical.com>
 * Copyright Canonical Ltd 2010. 
 * Licensed under GPLv3.
 */ 
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <fftw3.h>
#include <math.h>

// This function is (C) Lennart Poettering
static double linear_to_dB(double f) {
        return 20.0 * log10(f);
}


#define margin (10)

static void list_levels(double* arr, int count, int levelcount)
{
  int i,j,k;
  double curmax;

  for (k = 0; k < levelcount; k++) {    
    curmax = 0; j = -1;
    for (i = margin; i < count-margin; i++) 
      if (arr[i] > curmax) {
        curmax = arr[i];
        j = i;
      }
    
    if (j > -1) { 
      printf("%d %f %f\n", j, arr[j], linear_to_dB(arr[j]/32768));
      /* Clean out some smallies around that spot */
      for (i = j-margin; i < j+margin; i++)
        arr[i] = 0;
    }
  }
}

int main(int argc, char** argv)
{
  FILE *file;
  double* out = NULL;
  double* in = NULL;
  int N;
  fftw_plan p;

  if (argc != 2) {
    printf("Usage: powerfft <filename.raw>, where raw contains an array of doubles\n");
    return 0;
  }
  
  file = fopen(argv[1], "rb");
  if (file == NULL) {
    fprintf(stderr, "Could not open file %s\n", argv[1]);
    return errno;
  }

  // file size
  fseek(file, 0, SEEK_END); 
  N = ftell(file) / sizeof(double); 

  if (N == 0) {
    fprintf(stderr, "File is empty.");
    fclose(file);
    return EINVAL;
  }
  
/*  if (N > 30000) {
    N = 30000;
    fseek(file, -N*sizeof(double), SEEK_CUR);
  }
  else*/
  fseek(file, 0, SEEK_SET); 


  in = (double*) fftw_malloc(sizeof(double) * N);
  out = (double*) fftw_malloc(sizeof(double) * N);
  if (in == NULL || out == NULL) {
    fprintf(stderr, "Could not allocate memory\n");
    fclose(file);
    return ENOMEM;
  }

  p = fftw_plan_r2r_1d(N, in, out, FFTW_R2HC, FFTW_ESTIMATE);
  {
    int err = fread(in, sizeof(double), N, file);
    if (err != N) {
      fprintf(stderr, "Could not read file, err=%d, errno=%d\n", err, errno);
      return 0;
    }
  }
  fclose(file);
         
  fftw_execute(p); 

  for (int i=0; i < N/2-1; i++)
    out[i] = fabs(sqrt(out[i]*out[i] + out[N-i]*out[N-i]));

  list_levels(out, N/2-1, 10);

  fftw_destroy_plan(p);
  fftw_free(in); 
  fftw_free(out);
  return 0;
}

#if 0
Copy-paste - the ultimate version control system! ;-)

static size_t load_file(const char* filename, double** contents)
{
  size_t size;
  FILE *file = fopen(filename, "rb");
  *contents = NULL;
  if (file == NULL) {
    printf("Could not open file\n");
    return 0;
  }

  fseek(file, 0, SEEK_END); // seek to end of file
  size = ftell(file); // get current file pointer
  fseek(file, 0, SEEK_SET); // seek back to beginning of file
  *contents = fftw_malloc(size);
  if (*contents == NULL) {
    fclose(file);
    printf("Could not allocate memory\n");
    return 0;
  }

  if (fread(*contents, 1, size, file) != size) {
    printf("Could not read file\n");
    return 0;
  }
  fclose(file);
  return size / sizeof(double);
}

int main(int argc, char** argv)
{
  double* out = NULL;
  double* in = NULL;
  int N;
  fftw_plan p;

  if (argc != 2) {
    printf("Usage: powerfft <filename.raw>, where raw contains an array of doubles\n");
    return 1;
  }
  
  N = load_file(argv[1], &in);
  if (N == 0) {
     return 1;
  }

  out = (fftw_complex*) fftw_malloc(sizeof(double) * N);
  if (out == NULL) {
    printf("Could not allocate memory\n");
    return -ENOMEM;
  }

  p = fftw_plan_r2r_1d(N, in, out, FFTW_R2HC, FFTW_ESTIMATE);
         
  fftw_execute(p); /* repeat as needed */

  for (int i=0; i < N; i++)
    printf("%d: %f\n", i, out[i]);

  fftw_destroy_plan(p);
  fftw_free(in); 
  fftw_free(out);
  return 0;
}
#endif
